@@ -5,8 +5,7 @@ export the results of Epigrass simulations to the formats supported by these lib
 copyright 2007,2012 by Flavio Codeco Coelho
 Licensed under the GPL.
 """
-from __future__ import absolute_import
-from __future__ import print_function
+
 import locale, os, pylab
 from osgeo import ogr, osr, gdal
 from xml.dom import minidom, Node
@@ -15,11 +14,12 @@ from matplotlib.colors import Normalize
 from matplotlib import cm
 from numpy import array
 import geopandas as gpd
+import fiona
 from zipfile import ZipFile
 import json
-import six
-from six.moves import range
-from six.moves import zip
+import copy
+from shapely.geometry import Point, LineString
+
 
 class NewWorld:
     def __init__(self, filename, namefield, geocfield, outdir='.'):
@@ -33,7 +33,184 @@ class NewWorld:
         self.geocfield = geocfield
         self.namefield = namefield
         self.outdir = outdir
-        self.map = gpd.read_file(filename)
+        self.filename = filename
+        self.centdict = {}
+        self.namedict = {}
+        self.nlist = []
+        self.nodesource = None
+        self.edgesource = None
+        self.datasource = None
+        self.layers = fiona.listlayers(self.filename)
+        self.get_node_list()
+        with fiona.open(filename) as source:
+            self.driver = source.driver
+            self.crs = source.crs
+            self.map = gpd.read_file(filename, layer=source.name)
+        # self._check_map()
+
+    @property
+    def nodes(self):
+        if not self.nodesource:
+            self.create_node_layer()
+        return gpd.read_file(os.path.join(self.outdir, 'Nodes.gpkg'))
+
+    @property
+    def edges(self):
+        if not self.edgesource:
+            self.create_edge_layer()
+        return gpd.read_file(os.path.join(self.outdir, 'Edges.gpkg'))
+
+    @property
+    def data(self):
+        if not self.datasource:
+            self.create_data_layer()
+        return gpd.read_file(os.path.join(self.outdir, 'Data.gpkg'))
+
+    def _check_map(self):
+        map_size = len(self.map)
+        unique_gcs = len(self.map[self.geocfield].value_counts())
+        try:
+            assert map_size == unique_gcs
+        except AssertionError as e:
+            print(f"Column {self.geocfield} should have {map_size} unique values, but it has {unique_gcs}.")
+            raise(e)
+
+
+    def get_layer_list(self):
+        return self.layers
+
+    def draw_layer(self, l):
+        if l in self.layers:
+            df = gpd.read_file(self.filename, layer=l)
+            df.plot()
+            pylab.show()
+        else:
+            print(f'Layer {l} not available.')
+
+    def get_hex_color(self, value, cmap="YlOrRd" ):
+        cols = cm.get_cmap(cmap, 256)
+        rgba = cols(value * 256)
+        bgrcol = list(rgba[:-1])  # rgb(list)
+        bgrcol.reverse()  # turn it into bgr
+        hexcol = "#80" + rgb2hex(bgrcol)[1:]  # abgr Alpha set to 128
+        return hexcol
+
+    def get_node_list(self, l=None):
+        '''
+        Updates self.centdict with centroid coordinates and self.nlist with layer features
+        l is an OGR layer.
+        '''
+        if l is None:
+            l = self.layers[0]
+        df = gpd.read_file(self.filename, layer=l)
+        df[self.geocfield] = df[self.geocfield].astype(int)
+        for i, row in df.iterrows():
+            self.centdict[row[self.geocfield]] = (row['geometry'].centroid.x, row['geometry'].centroid.y, 0)
+            self.namedict[row[self.geocfield]] = row[self.namefield]
+            self.nlist.append(row)
+        return self.nlist
+
+    def create_node_layer(self):
+        """
+        Creates a new geopackage file to represent network nodes.
+        The node layer will be based on the centroids of the
+        polygons belonging to the map layer associated with this
+        world instance.
+        """
+        if os.path.exists(os.path.join(self.outdir, 'Nodes.gpkg')):
+            os.remove(os.path.join(self.outdir, 'Nodes.gpkg'))
+        if not self.centdict:
+            self.get_node_list()
+        if not self.nodesource:
+            centroids = self.map.geometry.centroid
+            nodes = gpd.GeoDataFrame(columns=[self.geocfield, self.namefield, 'x', 'y'])
+            for gc, xy in self.centdict.items():
+                nodes = nodes.append({self.geocfield: gc,
+                                      self.namefield: self.namedict[gc],
+                                      'x': xy[0],
+                                      'y': xy[1]
+                                      }, ignore_index=True)
+
+            nodes['geometry'] = gpd.points_from_xy(nodes.x, nodes.y)
+            nodes.to_file(os.path.join(self.outdir, 'Nodes.gpkg'), driver='GPKG')
+            dsn = fiona.open(os.path.join(self.outdir, 'Nodes.gpkg'))
+            self.nodesource = dsn
+        else:
+            pass
+
+    def create_edge_layer(self, elist):
+        """
+        Creates a new layer with edge information.
+        elist is a list of tuples:
+        (sgeoc,dgeoc,fsd,fds)
+        """
+        # Creates a new shape file to hold the data
+        if os.path.exists(os.path.join(self.outdir, 'Edges.gpkg')):
+            os.remove(os.path.join(self.outdir, 'Edges.gpkg'))
+
+        if not self.edgesource:
+            edges = gpd.GeoDataFrame(columns=['source_geocode', 'dest_geocode', 'flow_sd', 'flow_ds', 'geometry'])
+            elist = array(elist)
+            edges['source_geocode'] = elist[:, 0].astype(int)
+            edges['dest_geocode'] = elist[:, 1].astype(int)
+            edges['flow_sd'] = elist[:, 2].astype(float)
+            edges['flow_ds'] = elist[:, 3].astype(float)
+            for i, row in edges.iterrows():
+                line = LineString([self.centdict[row.source_geocode], self.centdict[row.dest_geocode]])
+                edges.at[i, 'geometry'] = line
+            edges.to_file(os.path.join(self.outdir, 'Edges.gpkg'), driver='GPKG')
+            self.edgesource = fiona.open(os.path.join(self.outdir, 'Edges.gpkg'))
+
+        else:
+            pass
+
+    def create_data_layer(self, varlist, data):
+        """
+        Creates a new layer to contain data about nodes, and saves it as a GPKG file
+
+        :param varlist: List of fields names associated with
+        the nodes: `['field1', field2, ...]`
+        :param data: List of lists whose first element is the geocode
+        and the remaining elements are values of the fields, in the
+        same order as they appear in varlist: `[[geocode, 'field1', field2, ...],...]`
+        """
+        data = array(data)
+        # building normalizers for each variable, except geocode
+        norms = [Normalize(c.min(), c.max()) for c in data[:, 1:].T]
+        if os.path.exists(os.path.join(self.outdir, 'Data.gpkg')):
+            os.remove(os.path.join(self.outdir, 'Data.gpkg'))
+        # Creates a new shape file to hold the data
+        if not self.datasource:
+            dsd = gpd.GeoDataFrame(columns=['geocode', 'name', 'colors']+varlist+['geometry'])
+
+            dsd['geocode'] = data[:, 0]
+            dsd['name'] = [self.namedict[int(gc)] for gc in data[:, 0]]
+            dsd['colors'] = [str([self.get_hex_color(norms[i](v)) for i, v in enumerate(l[1:])]) for l in data]
+            for i, v in enumerate(varlist):
+                dsd[v] = data[:, i+1]
+            for i, row in dsd.iterrows():
+                geom = self.map[self.map[self.geocfield] == row.geocode].geometry.values[0]
+                dsd.at[i, 'geometry'] = geom
+            dsd.to_file(os.path.join(self.outdir, 'Data.gpkg'), driver='GPKG')
+            self.datasource = fiona.open(os.path.join(self.outdir, 'Data.gpkg'))
+        else:
+            pass
+
+    def gen_sites_file(self, fname):
+        """
+        This method generate a sites
+        csv file from the nodes extracted from the
+        map.
+        """
+        if not self.nodesource:
+            self.create_node_layer()
+        df = copy.copy(self.nodes)
+        fields = copy.copy(self.data)
+        del df['geometry']
+        field_list = [c for c in fields.columns if c not in ['geocode', 'name', 'colors']]
+        gpd.concat([df, fields[field_list]], axis=1)
+        df.to_csv(fname, index=False)
+        return df
 
 
 class World:
@@ -50,7 +227,8 @@ class World:
         try:
             self.ds = ogr.Open(filename)
         except AttributeError:
-            raise AttributeError("Could not open the Shapefile {}. Please check if the path is correct.".format(filename))
+            raise AttributeError(
+                "Could not open the Shapefile {}. Please check if the path is correct.".format(filename))
         self.name = self.ds.GetName()
         self.driver = self.ds.GetDriver()
         layer = self.ds.GetLayer()
@@ -465,7 +643,7 @@ class AnimatedKML(object):
             ts.appendChild(w)
             pm_newtime.appendChild(ts)
             self.doc.appendChild(pm_newtime)
-        for pm in six.itervalues(self.pmdict):
+        for pm in self.pmdict.values():
             self.doc.removeChild(pm)
         self.pmdict = {}
 
