@@ -4,13 +4,11 @@ Model Management and simulation objects.
 """
 from __future__ import absolute_import
 from __future__ import print_function
-import pickle
 import time
 from copy import deepcopy
-from numpy import *
 from collections import OrderedDict
 from argparse import ArgumentParser
-import six.moves.configparser
+from configparser import ConfigParser
 import json
 import sqlite3
 import os
@@ -20,18 +18,24 @@ import pandas as pd
 import pymysql.cursors
 import numpy as np
 from Epigrass.simobj import graph, edge, siteobj
+from Epigrass import simobj
 from Epigrass import spread
 from Epigrass.data_io import *
 from Epigrass import epigdal
+from Epigrass import report
 from Epigrass import __version__
 import requests
-import six
-from six.moves import range
-from six.moves import zip
-from six.moves import input
+import hashlib
+import redis
+import pickle
+import multiprocessing
+
+redisclient = redis.StrictRedis()
 
 
-class simulate:
+
+
+class Simulate:
     """
     This class takes care of setting up the model, simulating it, and storing the results
     """
@@ -83,7 +87,7 @@ class simulate:
         """
         config = {}
         config = config.copy()
-        cp = six.moves.configparser.SafeConfigParser()
+        cp = ConfigParser()
         cp.read(fname)
         for sec in cp.sections():
             name = sec.lower()
@@ -98,7 +102,7 @@ class simulate:
         try:
             # WORLD
             # load Shapefile if specified
-            if config['the world.shapefile']:
+            if config.get('the world.shapefile', False):
                 self.shapefile = eval(config['the world.shapefile'])
             else:
                 self.shapefile = []
@@ -138,8 +142,8 @@ class simulate:
             self.steps = eval(config['simulation and output.steps'])
             self.outdir = config['simulation and output.outdir']
             self.SQLout = eval(config['simulation and output.sqlout'])
-            #            self.Rep = eval(config['simulation and output.report'])
-            #            self.siteRep = eval(config['simulation and output.siterep'])
+            self.Rep = eval(config['simulation and output.report'])
+            self.siteRep = eval(config['simulation and output.siterep'])
             self.replicas = eval(config['simulation and output.replicas'])
             self.randomize_seeds = eval(config['simulation and output.randseed'])
             self.Batch = eval(config['simulation and output.batch'])
@@ -148,7 +152,7 @@ class simulate:
             sys.exit("Please check the syntax of your '.epg' file.\nVariable %s, from section %s was not specified." % (
                 V[1], V[0]))
         if self.replicas:
-            #            self.Rep = 0 #Turns off reports
+            self.Rep = 0  # Turns off reports
             self.Batch = []  # Turns off Batch mode
             self.round = 0  # Initialize replicate counter
             # generate dictionaries for parameters and inits
@@ -170,19 +174,19 @@ class simulate:
         """
         Checks the type of the variables on the script
         """
-        self.Say("Checking syntax of model script... NOW")
+        print("Checking syntax of model script... NOW")
 
         if not os.access(self.sites, os.F_OK):
-            self.Say('Sites file %s does not exist, please check your script.' % self.sites)
+            print('Sites file %s does not exist, please check your script.' % self.sites)
             sys.exit()
         if not os.access(self.edges, os.F_OK):
-            self.Say('Egdes file %s does not exist, please check your script.' % self.edges)
+            print('Egdes file %s does not exist, please check your script.' % self.edges)
             sys.exit()
         if not self.modtype in ['SIS', 'SIS_s', 'SIR', 'SIR_s', 'SEIS', 'SEIS_s', 'SEIR', 'SEIR_s', 'SIpRpS',
                                 'SIpRpS_s', 'SIpR,SIpR_s', 'Influenza', 'Custom']:
-            self.Say('Model type %s is invalid, please check your script.' % self.modtype)
+            print('Model type %s is invalid, please check your script.' % self.modtype)
 
-        self.Say('Script %s passed syntax check NOW.' % self.modelName)
+        print('Script %s passed syntax check NOW.' % self.modelName)
 
     def deg2dec(self, coord):
         """
@@ -208,14 +212,14 @@ class simulate:
         for site in sitelist:
             if len(site) != ncols:
                 raise ValueError("This line in your sites file has a different number elements:\n%s" % str(site))
-            if ':' in site[0]:
-                lat = self.deg2dec(site[0])
-                longit = self.deg2dec(site[1])
+            if ':' in str(site[0]):
+                lat = self.deg2dec(str(site[0]))
+                longit = self.deg2dec(str(site[1]))
             else:
-                lat = float(site[0])
-                longit = float(site[1])
-            objlist.append(siteobj(site[2], site[3], (lat, longit), int((site[4]).strip()),
-                                   tuple([float(i.strip()) for i in site[5:]])))
+                lat = site[0]
+                longit = site[1]
+            objlist.append(siteobj(site[2], site[3], (lat, longit), int((site[4])),
+                                   tuple([float(i) for i in site[5:]])))
         for o in objlist:
             if self.stochTransp:
                 o.stochtransp = 1
@@ -253,7 +257,7 @@ class simulate:
 
                 for j in self.seed:
                     if int(o.geocode) == j[0] or self.seed[0][0] == "all":
-                        # self.Say("%s infected person(s) arrived at %s"%(j[2],o.sitename))
+                        # print("%s infected person(s) arrived at %s"%(j[2],o.sitename))
                         inits[j[1].lower()] += j[2]
                         o.createModel(self.modtype, self.modelName, v=values, bi=inits, bp=parms)
                     else:
@@ -281,19 +285,19 @@ class simulate:
                 continue
             for site in sitelist:
                 # print site.sitename
-                if str(site.geocode) == edg[5]:
+                if site.geocode == edg[5]:
                     source = site
                     # print 'found source = ',source.sitename
-                elif str(site.geocode) == edg[6]:
+                elif site.geocode == edg[6]:
                     dest = site
                     # print 'found dest = ',dest.sitename
                 if (source and dest):
                     # print 'next edge!'
                     break
             if not (source and dest):
-                # print source ,dest
+                print(type(edg[5]), type(edg[6]))
                 sys.exit(
-                    'One of the vertices on edge ' + edg[0] + '-' + edg[1] + ' could not be found on the site list')
+                    f'One of the vertices on edge {edg[0]}({edg[5]}) - {edg[1]}({edg[6]}) could not be found on the site list')
 
             objlist.append(edge(source, dest, edg[2], edg[3], float(edg[4])))
             source = dest = None
@@ -348,19 +352,17 @@ class simulate:
         '''
         # inits[self.seed[0][1].lower()] += 1
         seedvar = self.seed[0][1].lower()  # retrieve the name of the variable containing the seeds
-        self.Say('seedvar= %s' % seedvar)
+        print('seedvar= %s' % seedvar)
         site_list = self.g.site_list
         self.seed = [(seed.geocode, seedvar, n)]
 
         for site in site_list:
             if site.geocode == seed.geocode or seed.geocode == 'all':
                 site.bi[seedvar] = n
-                #                site.ts[0][seedpos] = n
 
-                self.Say("%s infected case(s) arrived at %s" % (n, seed.sitename))
+                print("%s infected case(s) arrived at %s" % (n, seed.sitename))
             else:
                 site.bi[seedvar] = 0
-                #                site.ts[0][seedpos] = 0
 
     def start(self):
         """
@@ -369,12 +371,12 @@ class simulate:
         if '/' in self.modelName:
             self.modelName = os.path.split(self.modelName)[-1]
         #        print "====>", self.gui
-        time.sleep(10)
-        self.Say('Simulation starting.')
+        # time.sleep(10)
+        print('Simulation starting.')
         start = time.time()
         self.runGraph(self.g, self.steps, transp=self.doTransp)
         elapsed = time.time() - start
-        self.Say('Simulation lasted %s seconds.' % elapsed)
+        print('Simulation lasted %s seconds.' % elapsed)
         if self.SQLout:
             if self.backend == 'csv':
                 self.outToCsv(self.modelName)
@@ -385,14 +387,14 @@ class simulate:
         self.dumpData()
         spread.Spread(self.g, self.outdir, self.encoding)
         # self.saveModel(self.modelName)
-        # pycallgraph.make_dot_graph(self.modelName+'_callgraph.png')
 
     def createDataFrame(self, site):
         """
         Saves the time series *site.ts* to outfile as specified on the
         model script.
         """
-        data = array(site.ts, float)
+        ts = array([eval(st) for st in redisclient.lrange(f'{site.geocode}:ts', 0, -1)])
+        data = array(ts, float)
         inc = site.incidence
         f = open(self.outdir + self.outfile, 'w')
         f.write('time,E,I,S,incidence\n')
@@ -412,44 +414,24 @@ class simulate:
         """
         if not self.World:
             return
-        # Initialize world output layers
-        # self.Say("Reading Nodes from shapefile...")
-        # self.World.get_node_list(self.World.ds.GetLayerByName(self.World.layerlist[0]))
-        # self.Say("Done reading nodes!")
-        # self.Say("Creating Nodes shapefile...")
-        # self.World.create_node_layer()
-        # self.Say("Done creating Nodes shapefile!")
-        # elist = [(gcs[0], gcs[1], sum(e.ftheta), sum(e.btheta)) for gcs, e in six.iteritems(self.g.edge_dict)]
-        # self.Say("Creating Edges shapefile...")
-        # self.World.create_edge_layer(elist)
-        # self.Say("Done creating Edges shapefile!")
-        # Generate site epidemic stats
+
         varlist = ["prevalence", "totalcases", "arrivals", "population"]
         sitestats = [(site.geocode, float(site.totalcases) / site.totpop, site.totalcases, sum(site.thetahist),
                       float(site.totpop)) for site in six.itervalues(self.g.site_dict)]
-        simdf = pd.DataFrame(data=array(sitestats), columns=[self.World.geocfield] + varlist)
+        # simdf = pd.DataFrame(data=array(sitestats), columns=[self.World.geocfield] + varlist)
         # print(self.World.map.info())
         self.World.map[self.World.geocfield] = self.World.map[self.World.geocfield].astype(int)
-        self.World.map = pd.merge(self.World.map, simdf, on=self.World.geocfield)
-        self.Say('Saving results in the map Data.gpkg')
-        self.World.map.to_file(os.path.join(self.outdir, 'Data.gpkg'))
-        # names = {k: v.sitename for k, v in six.iteritems(self.g.site_dict)}
-        #
-        # self.Say("Creating Data shapefile...")
-        # self.World.create_data_layer(varlist, sitestats)
-        # self.Say("Done creating Data shapefile!")
-        # # Generate the kml too.
-        # self.out_to_kml(names)
-        #
-        # self.Say("Done creating KML files!")
-        # # close files
-        # self.World.close_sources()
+        # self.World.map = pd.merge(self.World.map, simdf, on=self.World.geocfield)
+        print('Saving results in the map Data.gpkg')
+        self.World.create_data_layer(varlist, sitestats)
+
+
 
     def out_to_kml(self, names):
         """
         Generates output to kml files
         """
-        self.Say("Creating KML output files...")
+        print("Creating KML output files...")
         lr = self.World.datasource.GetLayer(0)
         k = epigdal.KmlGenerator()
         k.addNodes(lr, names)
@@ -460,22 +442,25 @@ class simulate:
         if len(site_dict) * len(list(site_dict.values())[0].ts) < 20000:  # Only reasonably sized animations
             for i, v in enumerate(list(site_dict.values())[0].vnames):
                 ka = epigdal.AnimatedKML(os.path.join(self.outdir, 'Data.kml'), extrude=True)
-                data = [(str(site.geocode), t, p[i]) for site in six.itervalues(site_dict) for t, p in
-                        enumerate(site.ts)]
+                data = []
+                for site in site_dict.values():
+                    ts = [eval(st) for st in redisclient.lrange(f'{site.geocode}:ts', 0, -1)]
+                    for t, p in enumerate(ts):
+                        data.append((str(site.geocode), t, p[i]))
                 ka.add_data(data)
                 ka.save(v + '_animation')
-                self.Say(v + '_animation')
+                print(v + '_animation')
                 del ka
         else:
-            self.Say("Simulation too large to export as kml.")
+            print("Simulation too large to export as kml.")
 
-        self.Say("Done creating KML files!")
+        print("Done creating KML files!")
 
     def series_to_JSON(self):
         """
         Saves timeseries to JSON for uploading to epigrass web
         """
-        self.Say('Saving series to JSON')
+        print('Saving series to JSON')
         series = {}
         for gc, s in six.iteritems(self.g.site_dict):
             length = max(map(len, s.ts))
@@ -516,7 +501,7 @@ class simulate:
         head = ['geocode', 'time', 'totpop', 'name',
                 'lat', 'longit']
         headerwritten = False
-        for site in six.itervalues(self.g.site_dict):
+        for site in self.g.site_dict.values():
             t = 0
             regb = [str(site.geocode), str(t), str(site.totpop),
                     site.sitename.replace('"', '').encode('ascii', 'replace'),
@@ -527,7 +512,8 @@ class simulate:
                     head.append('values%s' % n)
                     regb.append(str(v))
             # print site.sitename, site.ts
-            ts = array(site.ts[1:])  # remove init conds so that ts and inc are the same size
+            # ts = array(site.ts[1:])  # remove init conds so that ts and inc are the same size
+            ts = array([eval(st) for st in redisclient.lrange(f'{site.geocode}:ts', 0, -1)])
             head.extend(['incidence', 'arrivals'])
             for n, v in enumerate(site.vnames):
                 head.append(str(v))
@@ -607,9 +593,9 @@ class simulate:
         """
 
         if self.backend.lower() == "mysql":
-            self.Say('Saving data on MySQL...')
+            print('Saving data on MySQL...')
         elif self.backend.lower() == "sqlite":
-            self.Say('Saving data on SQLite...')
+            print('Saving data on SQLite...')
 
         con = None
         try:
@@ -625,22 +611,23 @@ class simulate:
                 con = sqlite3.connect("Epigrass.sqlite")
                 os.chdir(self.dir)
             # Define number of variables to be stored
-            nvar = len(list(self.g.site_dict.values())[0].ts[
-                           -1]) + 4  # state variables,  plus coords, plus incidence, plus infected arrivals.
+            nvar = len(list(self.g.site_dict.values())[
+                           0].vnames) + 4  # state variables,  plus coords, plus incidence, plus infected arrivals.
             str1 = '`%s` FLOAT(9),' * nvar  # nvar variables in the table
             str1lite = '%s REAL,' * nvar  # nvar variables in the SQLite table
             varnames = ['lat', 'longit'] + list(list(self.g.site_dict.values())[0].vnames) + ['incidence'] + [
                 'Arrivals']
             #            print nvar, varnames, str1
-            print((nvar, len(varnames)))
+            # print((nvar, len(varnames)))
+            # print(varnames)
             str1 = str1[:-1] % tuple(varnames)  # insert variable names (MySQL)
             str1lite = str1lite[:len(str1lite) - 1] % tuple(varnames)  # insert variable names (SQLITE)
             Cursor = con.cursor()
-            str2 = """CREATE TABLE %s(
+            str2 = f"""CREATE TABLE {table}(
             `geocode` INT( 9 )  ,
             `time` INT( 9 ) ,
             `name` varchar(128) ,
-            """ % table
+            """
             str2lite = """CREATE TABLE %s(
             geocode INTEGER  ,
             time INTEGER ,
@@ -659,12 +646,13 @@ class simulate:
                 str3 = str3[:-1] + ')'
             sql2 = 'INSERT INTO %s' % table + ' VALUES(' + str3
             nvalues = []
-            for site in six.itervalues(self.g.site_dict):
+            for site in self.g.site_dict.values():
                 geoc = site.geocode
                 lat = site.pos[0]
                 longit = site.pos[1]
                 name = site.sitename
-                ts = array(site.ts[1:])  # remove init conds so that ts and inc are the same size
+                # ts = array(site.ts[1:])  # remove init conds so that ts and inc are the same size
+                ts = array([eval(st) for st in redisclient.lrange(f'{geoc}:ts', 0, -1)])
                 inc = site.incidence
                 thist = site.thetahist
                 t = 0
@@ -723,7 +711,7 @@ class simulate:
         # saving the adjacency  matrix
         codeslist = [str(i.geocode) for i in six.itervalues(self.g.site_dict)]
         if not os.path.exists('adjmat.csv'):
-            self.Say('Saving the adjacency  matrix...')
+            print('Saving the adjacency  matrix...')
             am = self.g.getConnMatrix()
             amf = open('adjmat.csv', 'w')
             amf.write(','.join(codeslist) + '\n')
@@ -731,13 +719,13 @@ class simulate:
                 row = [str(i) for i in row]
                 amf.write(','.join(row) + '\n')
             amf.close()
-            self.Say('Done!')
+            print('Done!')
 
     def dumpData(self):
         """
         Dumps data as csv (comma-separated-values)
         """
-        self.Say("Starting simulation Analysis")
+        print("Starting simulation Analysis")
         curdir = os.getcwd()
         if not self.outdir == curdir:
             os.chdir(self.outdir)
@@ -746,7 +734,7 @@ class simulate:
         self.criaAdjMatrix()
         # saving the shortest path matrices
         #        if not os.path.exists('spmat.csv'):
-        #            self.Say('Calculating the shortest path matrices...')
+        #            print('Calculating the shortest path matrices...')
         #            ap = self.g.getAllPairs()
         #            f = open('ap','w')
         #            pickle.dump(ap,f)
@@ -767,7 +755,7 @@ class simulate:
         #            print 'Done!'
 
         # Saving epidemic path
-        self.Say('Saving Epidemic path...')
+        print('Saving Epidemic path...')
         if self.round:
             epp = codecs.open('epipath%s.csv' % str(self.round), 'w', self.encoding)
         else:
@@ -785,13 +773,13 @@ class simulate:
             else:
                 mli = 'NA'
             # print i[1].sitename, type(i[1].sitename), mli
-            epp.write(str(i[0]) + ',' + site.sitename + ',' + mli + '\n')
+            epp.write(f"{i[0]},{site.sitename},{mli}\n")
         epp.close()
-        self.Say('Done!')
+        print('Done!')
         self.g.save_topology('network.gexf')
 
         # saving Epistats
-        self.Say('Saving Epidemiological results...')
+        print('Saving Epidemiological results...')
         stats = [str(i) for i in self.g.getEpistats()]
 
         seed = \
@@ -814,10 +802,10 @@ class simulate:
         #        sstats = '%s,%s,%s,%s,%s'%(scent,sbetw,sthidx,sdeg,spop)
         stf.write(str(self.seed[0][0]) + ',' + sname + ',' + ','.join(stats) + ',' + sstats + '\n')
         stf.close()
-        self.Say('Done saving!')
+        print('Done saving!')
 
         # Saving site stats
-        self.Say('Saving site statistics...')
+        print('Saving site statistics...')
         # self.g.getAllPairs()
         if os.path.exists('sitestats.csv'):
             sitef = codecs.open('sitestats.csv', 'a', self.encoding)  # append to file
@@ -842,14 +830,13 @@ class simulate:
                 it = 'NA'
 
             #            sitef.write(str(self.round)+','+str(s.geocode)+','+s.sitename+','+it+','+degree+','+central+','+bet+','+thidx+','+distseed+','+seedgc+','+seedname+'\n')
-            sitef.write(str(self.round) + ',' + str(
-                s.geocode) + ',' + s.sitename + ',' + it + ',' + degree + ',' + seedgc + ',' + seedname + '\n')
+            sitef.write(f"{self.round},{s.geocode} ,{s.sitename} ,{it}, {degree}, {seedgc}, {seedname}\n")
 
         # Saving series to JSON
-        self.series_to_JSON()
+        # self.series_to_JSON()
         os.chdir(curdir)
 
-        self.Say('Done saving data!')
+        print('Done saving data!')
 
     def saveModel(self, fname):
         """
@@ -866,15 +853,19 @@ class simulate:
         g = pickle.load(fname)
         return g
 
-    def runGraph(self, graphobj, iterations=1, transp=0):
+    def runGraph(self, graphobj, iterations=1, transp=1):
         """
         Starts the simulation on a graph.
+        :param graphobj: The graph to run
+        :param iterations: how many time steps to simulate
+        :param transp: include the flow in the simulation
         """
+
         g = graphobj
         g.maxstep = iterations
         sites = list(graphobj.site_dict.values())
         edges = list(graphobj.edge_dict.values())
-        #        g.sites_done = 0
+
 
         if transp:
             for n in tqdm(range(iterations), desc='Simulation steps'):
@@ -883,37 +874,38 @@ class simulate:
                 results = [i.runModel(self.parallel) for i in sites]
                 if self.parallel:
                     [r.wait() for r in results]
-                for j in edges:
-                    j.migrate()
-                if self.gui:
-                    self.gui.stepLCD.display(g.simstep)
-                    self.gui.app.processEvents()
+                    # flows = PO.map(migrate, edges)
+                    for j in edges:
+                        j.migrate()
+                else:
+                    for j in edges:
+                        j.migrate()
+
                 g.simstep += 1
                 g.sites_done = 0
         else:
             for n in tqdm(range(iterations), desc='Simulation steps'):
+                results = []
                 for i in tqdm(sites, desc='Sites'):
-                    i.runModel(self.parallel)
-                if self.gui:
-                    self.gui.stepLCD.display(g.simstep)
-                    self.gui.app.processEvents()
-                    self.gui.RT.mutex.lock()
-                    self.gui.RT.emit(self.gui.QtCore.SIGNAL("drawStep"), g.simstep,
-                                     dict([(s.geocode, s.incidence[-1]) for s in sites]))
-                    self.gui.RT.mutex.unlock()
+                    results.append(i.runModel(self.parallel))
+                if self.parallel:
+                    [r.wait() for r in results]
+
                 g.simstep += 1
 
+
+
+
     def Say(self, string):
         """
         Exits outputs messages to the console or the gui accordingly
         """
-        if self.gui:
-            self.gui.textEdit1.insertPlainText(string + '\n''')
-        else:
-            if self.silent:
-                return
-            print()
-            string + '\n'
+        if self.silent:
+            return
+        print(string + '\r')
+
+def migrate(edge):
+    return edge.migrate()
 
 
 def storeSimulation(S, usr, passw, db='epigrass', host='localhost', port=3306):
@@ -939,13 +931,15 @@ def onStraightRun(args):
     Runs the model from the commandline
     """
     if args.backend == "mysql":
-        S = simulate(fname=args.epg[0], host=args.dbhost, user=args.dbuser, password=args.dbpass, backend=args.backend)
+        S = Simulate(fname=args.epg[0], host=args.dbhost, user=args.dbuser, password=args.dbpass, backend=args.backend)
     else:
-        S = simulate(fname=args.epg[0], backend=args.backend)
+        S = Simulate(fname=args.epg[0], backend=args.backend)
     S.parallel = args.parallel
     if not S.replicas:
         S.start()
         spread.Spread(S.g)
+        R = report.Report(S)
+        R.Assemble(type=S.Rep)
     else:
         repRuns(S)
 
@@ -959,9 +953,9 @@ def onStraightRun(args):
             # delete the old graph object to save memory
             S.graph = None
             # Generates the simulation object
-            T = simulate(fname=i, host=S.host, user=S.usr, password=S.passw, backend=S.backend)
+            T = Simulate(fname=i, host=S.host, user=S.usr, password=S.passw, backend=S.backend)
 
-            S.Say('starting model %s' % i)
+            print('starting model %s' % i)
             T.start()  # Start the simulation
 
 
@@ -978,13 +972,13 @@ def repRuns(S):
     password = S.passw
     backend = S.backend
     nseeds = S.seed[0][2]  # number o individual to be used as seeds
-    S.Say("Replication type: ", randseed)
+    print("Replication type: ", randseed)
     if randseed:
         seeds = S.randomizeSeed(randseed)
     reps = S.replicas
     for i in range(reps):
-        S.Say("Starting replicate number %s" % i)
-        S = simulate(fname=fname, host=host, user=user, password=password, backend=backend)
+        print("Starting replicate number %s" % i)
+        S = Simulate(fname=fname, host=host, user=user, password=password, backend=backend)
         if randseed:
             S.setSeed(seeds[i], nseeds)
         S.round = i
@@ -1000,7 +994,7 @@ def upload_model(args):
     """
     username = input("Enter your epigrass Web User id:")
     passwd = getpass("Enter your Epigrass Web password:")
-    S = simulate(fname=args.epg[0], backend=args.backend)
+    S = Simulate(fname=args.epg[0], backend=args.backend)
 
     app_url = "http://app.epigrass.net/simulations/view/new/"  # creating the app id
     r = requests.get(app_url, auth=(username, passwd))
@@ -1027,7 +1021,7 @@ def upload_model(args):
 
 def main():
     # Options and Argument parsing for running model from the command line, without the GUI.
-    usage = "usage: PROG [options] your_model.epg"
+    usage = "usage: epirunner [options] your_model.epg"
     #    parser = OptionParser(usage=usage, version="%prog "+__version__.version)
     parser = ArgumentParser(usage=usage, description="Run epigrass models from the console",
                             prog="epirunner " + __version__.version)
@@ -1057,6 +1051,14 @@ def main():
     else:
         onStraightRun(args)
 
+def end_pools():
+    PO.close()
+    PO.terminate()
+    simobj.PO.close()
+    simobj.PO.terminate()
 
+PO = multiprocessing.Pool()
 if __name__ == '__main__':
+    import atexit
+    atexit.register(end_pools)
     main()
